from typing import Literal
from typing_extensions import TypedDict, NotRequired  # Required by pydantic under Python < 3.12


class RedisConfigDict(TypedDict):
    """TypedDict representing the RedisConfig structure."""

    host: str
    port: int
    db: int


class DiskCacheConfigDict(TypedDict):
    """TypedDict representing the DiskCacheConfig structure."""

    cache_dir: str


class RateLimitConfigDict(TypedDict):
    """TypedDict representing the RateLimitConfig structure."""

    enabled: bool
    backend: Literal["memory", "redis", "diskcache"]
    redis: NotRequired[RedisConfigDict]
    diskcache: NotRequired[DiskCacheConfigDict]
    default_rpm: int
    default_tpm: int


class ServerDict(TypedDict):
    """TypedDict representing the Server structure."""

    host: str
    port: int
    url: NotRequired[str]


class EndpointOptionDict(TypedDict):
    """TypedDict representing the model endpoint option structure."""

    endpoint_id: str
    model_id: str
    rpm: NotRequired[int]
    tpm: NotRequired[int]
    concurrent_requests: NotRequired[int]


class RequestMappingDict(TypedDict):
    """TypedDict representing custom request mapping rules."""

    method: NotRequired[str]
    path: NotRequired[str]
    headers: NotRequired[dict[str, str]]
    body_template: NotRequired[dict | list | str]
    query_template: NotRequired[dict | list | str]


class ResponseMappingDict(TypedDict):
    """TypedDict representing custom response mapping rules."""

    model_path: NotRequired[str]
    data_path: NotRequired[str]
    results_path: NotRequired[str]
    field_map: NotRequired[dict[str, str]]
    usage_map: NotRequired[dict[str, str]]


class ModelConfigDict(TypedDict):
    """TypedDict representing the model configuration structure."""

    id: str
    endpoints: list[str | EndpointOptionDict]
    function_call_available: NotRequired[bool]
    response_format_available: NotRequired[bool]
    native_multimodal: NotRequired[bool]
    context_length: NotRequired[int | None]
    max_output_tokens: NotRequired[int | None]


class RetrievalModelConfigDict(TypedDict):
    """TypedDict representing retrieval model configuration."""

    id: str
    endpoints: list[str | EndpointOptionDict]
    protocol: NotRequired[str | None]
    dimensions: NotRequired[int | None]
    default_top_n: NotRequired[int | None]
    request_mapping: NotRequired[RequestMappingDict | None]
    response_mapping: NotRequired[ResponseMappingDict | None]


class BackendSettingsDict(TypedDict):
    """TypedDict representing the BackendSettings structure."""

    models: dict[str, ModelConfigDict]
    default_endpoint: NotRequired[str | None]


class RetrievalBackendSettingsDict(TypedDict):
    """TypedDict representing retrieval backend settings."""

    models: dict[str, RetrievalModelConfigDict]
    default_endpoint: NotRequired[str | None]


class EndpointSettingDict(TypedDict):
    """TypedDict representing the EndpointSetting structure."""

    id: str
    api_base: NotRequired[str | None]
    api_key: NotRequired[str]
    region: NotRequired[str]
    response_api: NotRequired[bool]
    endpoint_type: NotRequired[
        Literal[
            "default",
            "openai",
            "openai_azure",
            "openai_vertex",
            "anthropic",
            "anthropic_vertex",
            "anthropic_bedrock",
        ]
    ]
    credentials: NotRequired[dict]
    is_azure: NotRequired[bool]
    is_vertex: NotRequired[bool]
    is_bedrock: NotRequired[bool]
    rpm: NotRequired[int]
    tpm: NotRequired[int]
    concurrent_requests: NotRequired[int]
    proxy: NotRequired[str]


class BackendsDict(TypedDict):
    """TypedDict representing all model backends in a single dictionary."""

    anthropic: NotRequired[BackendSettingsDict]
    deepseek: NotRequired[BackendSettingsDict]
    gemini: NotRequired[BackendSettingsDict]
    groq: NotRequired[BackendSettingsDict]
    local: NotRequired[BackendSettingsDict]
    minimax: NotRequired[BackendSettingsDict]
    mistral: NotRequired[BackendSettingsDict]
    moonshot: NotRequired[BackendSettingsDict]
    openai: NotRequired[BackendSettingsDict]
    qwen: NotRequired[BackendSettingsDict]
    yi: NotRequired[BackendSettingsDict]
    zhipuai: NotRequired[BackendSettingsDict]
    baichuan: NotRequired[BackendSettingsDict]
    stepfun: NotRequired[BackendSettingsDict]
    xai: NotRequired[BackendSettingsDict]
    ernie: NotRequired[BackendSettingsDict]


class SettingsV1Dict(TypedDict):
    """TypedDict representing the expected structure of the settings dictionary."""

    endpoints: list[EndpointSettingDict]
    token_server: NotRequired[ServerDict]
    rate_limit: NotRequired[RateLimitConfigDict]

    # V1 format: each model backend config
    anthropic: NotRequired[BackendSettingsDict]
    deepseek: NotRequired[BackendSettingsDict]
    gemini: NotRequired[BackendSettingsDict]
    groq: NotRequired[BackendSettingsDict]
    local: NotRequired[BackendSettingsDict]
    minimax: NotRequired[BackendSettingsDict]
    mistral: NotRequired[BackendSettingsDict]
    moonshot: NotRequired[BackendSettingsDict]
    openai: NotRequired[BackendSettingsDict]
    qwen: NotRequired[BackendSettingsDict]
    yi: NotRequired[BackendSettingsDict]
    zhipuai: NotRequired[BackendSettingsDict]
    baichuan: NotRequired[BackendSettingsDict]
    stepfun: NotRequired[BackendSettingsDict]
    xai: NotRequired[BackendSettingsDict]
    ernie: NotRequired[BackendSettingsDict]


class SettingsV2Dict(TypedDict):
    """TypedDict representing the expected structure of the settings dictionary."""

    VERSION: NotRequired[str]
    endpoints: list[EndpointSettingDict]
    token_server: NotRequired[ServerDict]
    rate_limit: NotRequired[RateLimitConfigDict]

    # V2 format: all model backend configs in a single dictionary
    backends: NotRequired[BackendsDict]
    embedding_backends: NotRequired[dict[str, RetrievalBackendSettingsDict]]
    rerank_backends: NotRequired[dict[str, RetrievalBackendSettingsDict]]


SettingsDict = SettingsV2Dict



from __future__ import annotations

from pydantic import BaseModel, Field


class EmbeddingData(BaseModel):
    index: int
    embedding: list[float]
    object: str = "embedding"
    text: str | None = None
    metadata: dict | None = None


class EmbeddingUsage(BaseModel):
    prompt_tokens: int | None = None
    total_tokens: int | None = None


class EmbeddingResponse(BaseModel):
    model: str
    data: list[EmbeddingData] = Field(default_factory=list)
    usage: EmbeddingUsage | None = None
    raw_response: dict | None = None


class RerankResult(BaseModel):
    index: int
    relevance_score: float
    document: str | dict | None = None
    id: str | None = None
    metadata: dict | None = None


class RerankUsage(BaseModel):
    search_units: int | None = None
    total_tokens: int | None = None


class RerankResponse(BaseModel):
    model: str
    results: list[RerankResult] = Field(default_factory=list)
    usage: RerankUsage | None = None
    raw_response: dict | None = None


__all__ = [
    "EmbeddingData",
    "EmbeddingUsage",
    "EmbeddingResponse",
    "RerankResult",
    "RerankUsage",
    "RerankResponse",
]



import time
import asyncio
import threading
import importlib
from collections import defaultdict
from abc import ABC, abstractmethod
from typing import cast


def _raise_optional_dependency_error(package_name: str, extra_name: str, exc: ModuleNotFoundError) -> None:
    raise ModuleNotFoundError(f"Optional dependency '{package_name}' is required for this feature. Install with: pip install vv-llm[{extra_name}]") from exc


class AsyncRateLimiterBackend(ABC):
    """Rate Limiter Backend Abstract Base Class"""

    @abstractmethod
    async def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1) -> tuple[bool, float]:
        """Returns (allowed, wait_time)"""
        pass


class SyncRateLimiterBackend(ABC):
    """Rate Limiter Backend Abstract Base Class"""

    @abstractmethod
    def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1) -> tuple[bool, float]:
        """Returns (allowed, wait_time)"""
        pass


class AsyncMemoryRateLimiter(AsyncRateLimiterBackend):
    """Async Memory Rate Limiter"""

    def __init__(self):
        self.windows = defaultdict(list)
        self.tokens = defaultdict(int)
        self.lock = asyncio.Lock()

    def _get_last_reset(self, key):
        return self.windows[key][0] if self.windows[key] else time.time()

    async def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1):
        async with self.lock:
            now = time.time()

            # RPM 检查
            window = self.windows[key]
            window = [t for t in window if t > now - 60]
            if len(window) >= rpm:
                return False, 60 - (now - window[0])

            # TPM 检查
            if self.tokens[key] + request_cost > tpm:
                return False, 60 - (now - self._get_last_reset(key))

            window.append(now)
            self.tokens[key] += request_cost
            self.windows[key] = window[-rpm:]
            return True, 0


class SyncMemoryRateLimiter(SyncRateLimiterBackend):
    """Sync Memory Rate Limiter"""

    def __init__(self):
        self.windows = defaultdict(list)
        self.tokens = defaultdict(int)
        self.lock = threading.Lock()

    def _get_last_reset(self, key):
        return self.windows[key][0] if self.windows[key] else time.time()

    def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1) -> tuple[bool, float]:
        """Sync Rate Limiter Check

        Args:
            key: Rate Limiter Key
            rpm: Requests per minute limit
            tpm: Tokens per minute limit
            request_cost: The number of tokens consumed by this request

        Returns:
            Tuple[bool, float]: (allowed, wait_time)
        """
        with self.lock:
            now = time.time()

            # RPM 检查
            window = self.windows[key]
            window = [t for t in window if t > now - 60]
            if len(window) >= rpm:
                return False, 60 - (now - window[0])

            # TPM 检查
            if self.tokens[key] + request_cost > tpm:
                return False, 60 - (now - self._get_last_reset(key))

            window.append(now)
            self.tokens[key] += request_cost
            self.windows[key] = window[-rpm:]
            return True, 0


REDIS_SCRIPT = """
local key = KEYS[1]
local rpm = tonumber(ARGV[1])
local tpm = tonumber(ARGV[2])
local cost = tonumber(ARGV[3])

-- 使用Redis服务器时间（精确到微秒）
local server_time = redis.call('TIME')
local now = tonumber(server_time[1]) + tonumber(server_time[2])/1000000

-- RPM限制检查
local rpm_key = key..'_rpm'
local elements = redis.call('LRANGE', rpm_key, 0, -1)
local valid_elements = {}
local min_valid_time = now - 60

-- 过滤过期时间戳
for _, ts in ipairs(elements) do
    local timestamp = tonumber(ts)
    if timestamp > min_valid_time then
        table.insert(valid_elements, timestamp)
    end
end
local valid_count = #valid_elements

-- 新增：自动清理过期时间戳
if valid_count > 0 then
    redis.call('DEL', rpm_key)
    for _, ts in ipairs(valid_elements) do
        redis.call('RPUSH', rpm_key, ts)
    end
    redis.call('EXPIRE', rpm_key, 60)
end

if valid_count >= rpm then
    local oldest = valid_elements[valid_count]  -- 最旧的有效时间戳
    local remaining = math.max(0.001, 60 - (now - oldest))  -- 保证最小等待时间
    return {0, math.ceil(remaining * 1000)/1000}  -- 保留3位小数
end

-- TPM限制检查（保持不变）
local tpm_key = key..'_tpm'
local current_tokens = tonumber(redis.call('GET', tpm_key) or 0)
if current_tokens + cost > tpm then
    local ttl = redis.call('TTL', tpm_key)
    if ttl < 0 then
        redis.call('SETEX', tpm_key, 60, current_tokens)
        ttl = 60
    end
    return {0, ttl}
end

-- 更新计数（增加时间戳覆盖写入）
redis.call('LPUSH', rpm_key, now)
redis.call('LTRIM', rpm_key, 0, rpm-1)
redis.call('EXPIRE', rpm_key, 60)

redis.call('INCRBY', tpm_key, cost)
redis.call('EXPIRE', tpm_key, 60)

return {1, 0}
"""


class AsyncRedisRateLimiter(AsyncRateLimiterBackend):
    """Async Redis Rate Limiter"""

    def __init__(self, host: str = "localhost", port: int = 6379, db: int = 0):
        try:
            import redis.asyncio as redis
        except ModuleNotFoundError as exc:
            _raise_optional_dependency_error("redis", "redis", exc)

        self.redis = redis.Redis(host=host, port=port, db=db)
        self.script = self.redis.register_script(REDIS_SCRIPT)

    async def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1):
        result = await self.script(keys=[key], args=[rpm, tpm, request_cost])
        allowed, wait_time = result
        return bool(allowed), max(1, float(wait_time))


class SyncRedisRateLimiter(SyncRateLimiterBackend):
    """Sync Redis Rate Limiter"""

    def __init__(self, host: str = "localhost", port: int = 6379, db: int = 0):
        try:
            import redis
        except ModuleNotFoundError as exc:
            _raise_optional_dependency_error("redis", "redis", exc)

        self.redis = redis.Redis(host=host, port=port, db=db)
        self.script = self.redis.register_script(REDIS_SCRIPT)

    def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1):
        result = self.script(keys=[key], args=[rpm, tpm, request_cost])
        allowed, wait_time = result
        return bool(allowed), max(1, float(wait_time))


class AsyncDiskCacheRateLimiter(AsyncRateLimiterBackend):
    """基于 diskcache 的异步限流器实现"""

    def __init__(self, cache_dir: str = ".rate_limit_cache"):
        """初始化 diskcache 限流器

        Args:
            cache_dir: 缓存目录路径
        """
        try:
            Cache = importlib.import_module("diskcache").Cache
        except ModuleNotFoundError as exc:
            _raise_optional_dependency_error("diskcache", "diskcache", exc)

        self.cache = Cache(cache_dir)
        self._lock = asyncio.Lock()

    def _get_rpm_key(self, key: str) -> str:
        return f"{key}_rpm"

    def _get_tpm_key(self, key: str) -> str:
        return f"{key}_tpm"

    async def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1) -> tuple[bool, float]:
        """检查是否超出限流阈值

        Args:
            key: 限流键
            rpm: 每分钟请求数限制
            tpm: 每分钟令牌数限制
            request_cost: 本次请求消耗的令牌数

        Returns:
            Tuple[bool, float]: (是否允许请求, 需要等待的时间)
        """
        async with self._lock:
            now = time.time()
            rpm_key = self._get_rpm_key(key)
            tpm_key = self._get_tpm_key(key)

            # RPM 检查
            window = cast(list[float], self.cache.get(rpm_key, []) or [])
            window = [t for t in window if t > now - 60]  # 清理过期时间戳

            if len(window) >= rpm:
                return False, 60 - (now - window[0])

            # TPM 检查
            current_tokens = cast(int, self.cache.get(tpm_key, 0))
            if current_tokens + request_cost > tpm:
                # 获取最早的请求时间
                oldest_time = window[0] if window else now
                return False, 60 - (now - oldest_time)

            # 更新状态
            window.append(now)
            window = window[-rpm:]  # 只保留最近的 rpm 个时间戳
            self.cache.set(rpm_key, window, expire=60)
            self.cache.set(tpm_key, current_tokens + request_cost, expire=60)

            return True, 0


class SyncDiskCacheRateLimiter(SyncRateLimiterBackend):
    """基于 diskcache 的同步限流器实现"""

    def __init__(self, cache_dir: str = ".rate_limit_cache"):
        """初始化 diskcache 限流器

        Args:
            cache_dir: 缓存目录路径
        """
        try:
            Cache = importlib.import_module("diskcache").Cache
        except ModuleNotFoundError as exc:
            _raise_optional_dependency_error("diskcache", "diskcache", exc)

        self.cache = Cache(cache_dir)
        self._lock = threading.Lock()

    def _get_rpm_key(self, key: str) -> str:
        return f"{key}_rpm"

    def _get_tpm_key(self, key: str) -> str:
        return f"{key}_tpm"

    def check_limit(self, key: str, rpm: int, tpm: int, request_cost: int = 1) -> tuple[bool, float]:
        """检查是否超出限流阈值

        Args:
            key: 限流键
            rpm: 每分钟请求数限制
            tpm: 每分钟令牌数限制
            request_cost: 本次请求消耗的令牌数

        Returns:
            Tuple[bool, float]: (是否允许请求, 需要等待的时间)
        """
        with self._lock:
            now = time.time()
            rpm_key = self._get_rpm_key(key)
            tpm_key = self._get_tpm_key(key)

            # RPM 检查
            window = cast(list[float], self.cache.get(rpm_key, []) or [])
            window = [t for t in window if t > now - 60]  # 清理过期时间戳

            if len(window) >= rpm:
                return False, 60 - (now - window[0])

            # TPM 检查
            current_tokens = cast(int, self.cache.get(tpm_key, 0))
            if current_tokens + request_cost > tpm:
                # 获取最早的请求时间
                oldest_time = window[0] if window else now
                return False, 60 - (now - oldest_time)

            # 更新状态
            window.append(now)
            window = window[-rpm:]  # 只保留最近的 rpm 个时间戳
            self.cache.set(rpm_key, window, expire=60)
            self.cache.set(tpm_key, current_tokens + request_cost, expire=60)

            return True, 0



from __future__ import annotations

from typing import Any, TYPE_CHECKING

import httpx

from ..settings import normalize_settings
from ..types.enums import RerankBackendType
from ..types.llm_parameters import EndpointSetting, ResponseMapping
from ..types.retrieval_parameters import RerankResponse, RerankResult, RerankUsage
from ..types.settings import SettingsDict
from ..utilities.gcp_token import get_token_with_cache
from ..retrieval_clients.common import BaseAsyncRetrievalClient, BaseRetrievalClient
from ..retrieval_clients.common import build_url, extract_json_path, render_template

if TYPE_CHECKING:
    from ..settings import Settings

DEFAULT_RERANK_PROTOCOLS: dict[str, str] = {
    RerankBackendType.Cohere.value: "cohere_rerank_v2",
    RerankBackendType.Jina.value: "jina_rerank_v1",
    RerankBackendType.Voyage.value: "voyage_rerank_v1",
    RerankBackendType.OpenAI.value: "custom_json_http",
    RerankBackendType.Local.value: "custom_json_http",
    RerankBackendType.Custom.value: "custom_json_http",
}


def _normalize_backend_name(backend: RerankBackendType | str) -> str:
    if isinstance(backend, RerankBackendType):
        return backend.value
    return str(backend).lower()


def _normalize_document(value: Any) -> str | dict | None:
    if value is None:
        return None
    if isinstance(value, dict):
        if set(value.keys()) == {"text"}:
            return value.get("text")
        return value
    if isinstance(value, str):
        return value
    return str(value)


def _usage_from_mapping(raw: dict[str, Any], response_mapping: ResponseMapping | None) -> RerankUsage | None:
    if response_mapping is None:
        return None
    search_units_path = response_mapping.usage_map.get("search_units")
    total_tokens_path = response_mapping.usage_map.get("total_tokens")

    search_units = None
    if search_units_path:
        search_units = extract_json_path(raw, search_units_path)
        if search_units is None:
            raise ValueError(
                f"Rerank response_mapping.usage_map.search_units path '{search_units_path}' not found in response."
            )

    total_tokens = None
    if total_tokens_path:
        total_tokens = extract_json_path(raw, total_tokens_path)
        if total_tokens is None:
            raise ValueError(
                f"Rerank response_mapping.usage_map.total_tokens path '{total_tokens_path}' not found in response."
            )

    if search_units is None and total_tokens is None:
        return None

    try:
        parsed_search_units = int(search_units) if search_units is not None else None
    except (TypeError, ValueError) as exc:
        raise ValueError(
            "Rerank response_mapping.usage_map.search_units must resolve to an integer-compatible value."
        ) from exc

    try:
        parsed_total_tokens = int(total_tokens) if total_tokens is not None else None
    except (TypeError, ValueError) as exc:
        raise ValueError(
            "Rerank response_mapping.usage_map.total_tokens must resolve to an integer-compatible value."
        ) from exc

    return RerankUsage(
        search_units=parsed_search_units,
        total_tokens=parsed_total_tokens,
    )


def _extract_item_field(item: Any, path: str | None):
    if path is None:
        return None
    if path.startswith("$"):
        return extract_json_path(item, path)
    return extract_json_path(item, f"$.{path}")


def _parse_result_list(
    *,
    items: list[Any],
    default_documents: list[str | dict],
    field_map: dict[str, str] | None = None,
) -> list[RerankResult]:
    field_map = field_map or {}
    results: list[RerankResult] = []

    for idx, item in enumerate(items):
        if not isinstance(item, dict):
            continue

        mapped_index = _extract_item_field(item, field_map.get("index"))
        mapped_score = _extract_item_field(item, field_map.get("relevance_score"))
        mapped_document = _extract_item_field(item, field_map.get("document"))
        mapped_id = _extract_item_field(item, field_map.get("id"))
        mapped_metadata = _extract_item_field(item, field_map.get("metadata"))

        index_field_path = field_map.get("index")
        if index_field_path and mapped_index is None:
            raise ValueError(
                f"Rerank response_mapping.field_map.index path '{index_field_path}' not found for item[{idx}]."
            )

        try:
            result_index = int(mapped_index) if mapped_index is not None else int(item.get("index", idx))
        except (TypeError, ValueError) as exc:
            raise ValueError(
                f"Rerank item[{idx}] index must be integer-compatible, got: {mapped_index!r}."
            ) from exc

        score = mapped_score
        score_field_path = field_map.get("relevance_score")
        if score_field_path and score is None:
            raise ValueError(
                f"Rerank response_mapping.field_map.relevance_score path '{score_field_path}' not found for item[{idx}]."
            )
        if score is None:
            score = item.get("relevance_score", item.get("score", 0.0))
        try:
            score_value = float(score)
        except (TypeError, ValueError) as exc:
            raise ValueError(
                f"Rerank item[{idx}] relevance_score must be numeric-compatible, got: {score!r}."
            ) from exc

        document = mapped_document
        if document is None:
            document = item.get("document")
        if document is None and 0 <= result_index < len(default_documents):
            document = default_documents[result_index]

        result_id = mapped_id if mapped_id is not None else item.get("id")
        metadata = mapped_metadata if mapped_metadata is not None else item.get("metadata")

        results.append(
            RerankResult(
                index=result_index,
                relevance_score=score_value,
                document=_normalize_document(document),
                id=str(result_id) if result_id is not None else None,
                metadata=metadata if isinstance(metadata, dict) else None,
            )
        )

    return results


def _parse_cohere_rerank(raw: dict[str, Any], model_id: str, documents: list[str | dict]) -> RerankResponse:
    items = raw.get("results", [])
    if not isinstance(items, list):
        items = []

    usage = None
    billed_units = raw.get("meta", {}).get("billed_units", {}) if isinstance(raw.get("meta"), dict) else {}
    if isinstance(billed_units, dict):
        search_units = billed_units.get("search_units")
        if search_units is not None:
            usage = RerankUsage(search_units=int(search_units))

    return RerankResponse(
        model=str(raw.get("model", model_id)),
        results=_parse_result_list(items=items, default_documents=documents),
        usage=usage,
        raw_response=raw,
    )


def _parse_jina_rerank(raw: dict[str, Any], model_id: str, documents: list[str | dict]) -> RerankResponse:
    items = raw.get("results", raw.get("data", []))
    if not isinstance(items, list):
        items = []

    usage_raw = raw.get("usage") if isinstance(raw.get("usage"), dict) else None
    usage = None
    if usage_raw is not None:
        total_tokens = usage_raw.get("total_tokens")
        usage = RerankUsage(total_tokens=int(total_tokens) if total_tokens is not None else None)

    return RerankResponse(
        model=str(raw.get("model", model_id)),
        results=_parse_result_list(items=items, default_documents=documents),
        usage=usage,
        raw_response=raw,
    )


def _parse_voyage_rerank(raw: dict[str, Any], model_id: str, documents: list[str | dict]) -> RerankResponse:
    items = raw.get("data", raw.get("results", []))
    if not isinstance(items, list):
        items = []

    usage = None
    usage_raw = raw.get("usage") if isinstance(raw.get("usage"), dict) else None
    if usage_raw is not None:
        total_tokens = usage_raw.get("total_tokens")
        usage = RerankUsage(total_tokens=int(total_tokens) if total_tokens is not None else None)

    return RerankResponse(
        model=str(raw.get("model", model_id)),
        results=_parse_result_list(items=items, default_documents=documents),
        usage=usage,
        raw_response=raw,
    )


def _parse_custom_rerank(
    raw: dict[str, Any],
    model_id: str,
    documents: list[str | dict],
    response_mapping: ResponseMapping | None,
) -> RerankResponse:
    results_path = response_mapping.results_path if response_mapping and response_mapping.results_path else "$.results[*]"
    items = extract_json_path(raw, results_path)

    if items is None and response_mapping and response_mapping.results_path:
        raise ValueError(f"Rerank response_mapping.results_path '{results_path}' not found in response.")

    if items is None:
        items = []
    if not isinstance(items, list):
        items = [items]

    model = model_id
    if response_mapping and response_mapping.model_path:
        mapped_model = extract_json_path(raw, response_mapping.model_path)
        if mapped_model is None:
            raise ValueError(
                f"Rerank response_mapping.model_path '{response_mapping.model_path}' not found in response."
            )
        model = str(mapped_model)

    field_map = response_mapping.field_map if response_mapping is not None else {}
    usage = _usage_from_mapping(raw, response_mapping)

    results = _parse_result_list(items=items, default_documents=documents, field_map=field_map)
    if not results:
        raise ValueError(
            f"Rerank response produced no results after applying mapping. results_path='{results_path}'."
        )

    return RerankResponse(
        model=model,
        results=results,
        usage=usage,
        raw_response=raw,
    )


def _default_auth_headers(endpoint: EndpointSetting) -> dict[str, str]:
    if endpoint.is_azure or endpoint.endpoint_type == "openai_azure":
        return {"api-key": endpoint.api_key or ""}
    if endpoint.endpoint_type == "openai_vertex":
        if endpoint.credentials is None:
            raise ValueError("OpenAI Vertex endpoint requires credentials")
        access_token, expires_at = get_token_with_cache(
            credentials=endpoint.credentials,
            proxy=endpoint.proxy,
            cached_token=endpoint.access_token,
            cached_expires_at=endpoint.access_token_expires_at,
        )
        endpoint.access_token = access_token
        endpoint.access_token_expires_at = expires_at
        return {"Authorization": f"Bearer {access_token}"}
    if endpoint.api_key:
        return {"Authorization": f"Bearer {endpoint.api_key}"}
    return {}


class RerankClient(BaseRetrievalClient):
    def __init__(
        self,
        backend: RerankBackendType | str,
        model: str | None = None,
        random_endpoint: bool = True,
        endpoint_id: str = "",
        http_client: httpx.Client | None = None,
        settings: Settings | SettingsDict | None = None,
    ):
        normalized_settings = normalize_settings(settings)
        backend_name = _normalize_backend_name(backend)
        backend_settings = normalized_settings.get_rerank_backend(backend_name)

        super().__init__(
            model=model or "",
            backend_name=backend_name,
            backend_settings=backend_settings,
            random_endpoint=random_endpoint,
            endpoint_id=endpoint_id,
            http_client=http_client,
            settings=normalized_settings,
        )

    def _switch_model(self, model: str):
        if model == self.model:
            return
        self.model = model
        self.model_setting = self.backend_settings.get_model_setting(model)
        self.model_id = self.model_setting.id
        self.endpoint = None

    def _request_json(
        self,
        *,
        endpoint: EndpointSetting,
        method: str,
        path: str,
        headers: dict[str, str] | None,
        body: dict | list | str | None,
        query: dict | None,
        timeout: float | httpx.Timeout | None,
    ) -> dict[str, Any]:
        request_headers = {"Content-Type": "application/json", **_default_auth_headers(endpoint)}
        if headers:
            request_headers.update(headers)

        client = self.http_client or httpx.Client(proxy=endpoint.proxy)
        should_close = self.http_client is None

        try:
            response = client.request(
                method=method.upper(),
                url=build_url(endpoint.api_base, path),
                headers=request_headers,
                params=query,
                json=body,
                timeout=timeout,
            )
            response.raise_for_status()
            return response.json()
        finally:
            if should_close:
                client.close()

    def _build_custom_request(
        self,
        *,
        model_id: str,
        query_text: str,
        documents: list[str | dict],
        top_n: int | None,
        return_documents: bool,
        extra_body: dict[str, Any] | None,
    ) -> tuple[str, str, dict[str, str] | None, dict | list | str | None, dict | None]:
        mapping = self.model_setting.request_mapping
        method = mapping.method if mapping else "POST"
        path = mapping.path if mapping else "/rerank"

        context = {
            "model": self.model,
            "model_id": model_id,
            "query": query_text,
            "documents": documents,
            "top_n": top_n,
            "return_documents": return_documents,
        }

        headers = render_template(mapping.headers, context) if mapping and mapping.headers else None
        body = render_template(mapping.body_template, context) if mapping and mapping.body_template is not None else None
        query = render_template(mapping.query_template, context) if mapping and mapping.query_template is not None else None

        if body is None:
            body = {
                "model": model_id,
                "query": query_text,
                "documents": documents,
                "top_n": top_n,
                "return_documents": return_documents,
            }
        if extra_body and isinstance(body, dict):
            body.update(extra_body)

        return method, path, headers, body, query

    def rerank(
        self,
        *,
        query: str,
        documents: list[str | dict],
        top_n: int | None = None,
        return_documents: bool = True,
        model: str | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> RerankResponse:
        if model is not None:
            self._switch_model(model)

        endpoint, model_id = self._set_endpoint()
        protocol = self.model_setting.protocol or DEFAULT_RERANK_PROTOCOLS.get(self.backend_name, "custom_json_http")

        effective_top_n = top_n if top_n is not None else self.model_setting.default_top_n
        request_body_for_rate = {
            "query": query,
            "documents": documents,
            "top_n": effective_top_n,
        }
        self._acquire_rate_limit(endpoint, request_body_for_rate)

        if protocol == "cohere_rerank_v2":
            body = {
                "model": model_id,
                "query": query,
                "documents": documents,
                "return_documents": return_documents,
            }
            if effective_top_n is not None:
                body["top_n"] = effective_top_n
            if extra_body:
                body.update(extra_body)
            raw = self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/v2/rerank",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_cohere_rerank(raw, model_id, documents)

        if protocol == "jina_rerank_v1":
            body = {
                "model": model_id,
                "query": query,
                "documents": documents,
                "return_documents": return_documents,
            }
            if effective_top_n is not None:
                body["top_n"] = effective_top_n
            if extra_body:
                body.update(extra_body)
            raw = self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/v1/rerank",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_jina_rerank(raw, model_id, documents)

        if protocol == "voyage_rerank_v1":
            body = {
                "model": model_id,
                "query": query,
                "documents": documents,
                "return_documents": return_documents,
            }
            if effective_top_n is not None:
                body["top_k"] = effective_top_n
            if extra_body:
                body.update(extra_body)
            raw = self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/rerank",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_voyage_rerank(raw, model_id, documents)

        if protocol == "custom_json_http":
            method, path, headers, body, req_query = self._build_custom_request(
                model_id=model_id,
                query_text=query,
                documents=documents,
                top_n=effective_top_n,
                return_documents=return_documents,
                extra_body=extra_body,
            )
            raw = self._request_json(
                endpoint=endpoint,
                method=method,
                path=path,
                headers=headers,
                body=body,
                query=req_query if isinstance(req_query, dict) else None,
                timeout=timeout,
            )
            return _parse_custom_rerank(raw, model_id, documents, self.model_setting.response_mapping)

        raise ValueError(f"Unsupported rerank protocol: {protocol}")

    def create_rerank(
        self,
        *,
        query: str,
        documents: list[str | dict],
        top_n: int | None = None,
        return_documents: bool = True,
        model: str | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> RerankResponse:
        return self.rerank(
            query=query,
            documents=documents,
            top_n=top_n,
            return_documents=return_documents,
            model=model,
            extra_body=extra_body,
            timeout=timeout,
        )


class AsyncRerankClient(BaseAsyncRetrievalClient):
    def __init__(
        self,
        backend: RerankBackendType | str,
        model: str | None = None,
        random_endpoint: bool = True,
        endpoint_id: str = "",
        http_client: httpx.AsyncClient | None = None,
        settings: Settings | SettingsDict | None = None,
    ):
        normalized_settings = normalize_settings(settings)
        backend_name = _normalize_backend_name(backend)
        backend_settings = normalized_settings.get_rerank_backend(backend_name)

        super().__init__(
            model=model or "",
            backend_name=backend_name,
            backend_settings=backend_settings,
            random_endpoint=random_endpoint,
            endpoint_id=endpoint_id,
            http_client=http_client,
            settings=normalized_settings,
        )

    def _switch_model(self, model: str):
        if model == self.model:
            return
        self.model = model
        self.model_setting = self.backend_settings.get_model_setting(model)
        self.model_id = self.model_setting.id
        self.endpoint = None

    async def _request_json(
        self,
        *,
        endpoint: EndpointSetting,
        method: str,
        path: str,
        headers: dict[str, str] | None,
        body: dict | list | str | None,
        query: dict | None,
        timeout: float | httpx.Timeout | None,
    ) -> dict[str, Any]:
        request_headers = {"Content-Type": "application/json", **_default_auth_headers(endpoint)}
        if headers:
            request_headers.update(headers)

        client = self.http_client or httpx.AsyncClient(proxy=endpoint.proxy)
        should_close = self.http_client is None

        try:
            response = await client.request(
                method=method.upper(),
                url=build_url(endpoint.api_base, path),
                headers=request_headers,
                params=query,
                json=body,
                timeout=timeout,
            )
            response.raise_for_status()
            return response.json()
        finally:
            if should_close:
                await client.aclose()

    def _build_custom_request(
        self,
        *,
        model_id: str,
        query_text: str,
        documents: list[str | dict],
        top_n: int | None,
        return_documents: bool,
        extra_body: dict[str, Any] | None,
    ) -> tuple[str, str, dict[str, str] | None, dict | list | str | None, dict | None]:
        mapping = self.model_setting.request_mapping
        method = mapping.method if mapping else "POST"
        path = mapping.path if mapping else "/rerank"

        context = {
            "model": self.model,
            "model_id": model_id,
            "query": query_text,
            "documents": documents,
            "top_n": top_n,
            "return_documents": return_documents,
        }

        headers = render_template(mapping.headers, context) if mapping and mapping.headers else None
        body = render_template(mapping.body_template, context) if mapping and mapping.body_template is not None else None
        query = render_template(mapping.query_template, context) if mapping and mapping.query_template is not None else None

        if body is None:
            body = {
                "model": model_id,
                "query": query_text,
                "documents": documents,
                "top_n": top_n,
                "return_documents": return_documents,
            }
        if extra_body and isinstance(body, dict):
            body.update(extra_body)

        return method, path, headers, body, query

    async def rerank(
        self,
        *,
        query: str,
        documents: list[str | dict],
        top_n: int | None = None,
        return_documents: bool = True,
        model: str | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> RerankResponse:
        if model is not None:
            self._switch_model(model)

        endpoint, model_id = self._set_endpoint()
        protocol = self.model_setting.protocol or DEFAULT_RERANK_PROTOCOLS.get(self.backend_name, "custom_json_http")

        effective_top_n = top_n if top_n is not None else self.model_setting.default_top_n
        request_body_for_rate = {
            "query": query,
            "documents": documents,
            "top_n": effective_top_n,
        }
        await self._acquire_rate_limit(endpoint, request_body_for_rate)

        if protocol == "cohere_rerank_v2":
            body = {
                "model": model_id,
                "query": query,
                "documents": documents,
                "return_documents": return_documents,
            }
            if effective_top_n is not None:
                body["top_n"] = effective_top_n
            if extra_body:
                body.update(extra_body)
            raw = await self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/v2/rerank",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_cohere_rerank(raw, model_id, documents)

        if protocol == "jina_rerank_v1":
            body = {
                "model": model_id,
                "query": query,
                "documents": documents,
                "return_documents": return_documents,
            }
            if effective_top_n is not None:
                body["top_n"] = effective_top_n
            if extra_body:
                body.update(extra_body)
            raw = await self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/v1/rerank",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_jina_rerank(raw, model_id, documents)

        if protocol == "voyage_rerank_v1":
            body = {
                "model": model_id,
                "query": query,
                "documents": documents,
                "return_documents": return_documents,
            }
            if effective_top_n is not None:
                body["top_k"] = effective_top_n
            if extra_body:
                body.update(extra_body)
            raw = await self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/rerank",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_voyage_rerank(raw, model_id, documents)

        if protocol == "custom_json_http":
            method, path, headers, body, req_query = self._build_custom_request(
                model_id=model_id,
                query_text=query,
                documents=documents,
                top_n=effective_top_n,
                return_documents=return_documents,
                extra_body=extra_body,
            )
            raw = await self._request_json(
                endpoint=endpoint,
                method=method,
                path=path,
                headers=headers,
                body=body,
                query=req_query if isinstance(req_query, dict) else None,
                timeout=timeout,
            )
            return _parse_custom_rerank(raw, model_id, documents, self.model_setting.response_mapping)

        raise ValueError(f"Unsupported rerank protocol: {protocol}")

    async def create_rerank(
        self,
        *,
        query: str,
        documents: list[str | dict],
        top_n: int | None = None,
        return_documents: bool = True,
        model: str | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> RerankResponse:
        return await self.rerank(
            query=query,
            documents=documents,
            top_n=top_n,
            return_documents=return_documents,
            model=model,
            extra_body=extra_body,
            timeout=timeout,
        )


__all__ = [
    "RerankClient",
    "AsyncRerankClient",
]



# @Author: Bi Ying
# @Date:   2024-07-27 00:02:34
from typing import Final
from typing_extensions import TypedDict, NotRequired

from .enums import ContextLengthControlType

CONTEXT_LENGTH_CONTROL: Final[ContextLengthControlType] = ContextLengthControlType.Latest

ENDPOINT_CONCURRENT_REQUESTS: Final[int] = 20
ENDPOINT_RPM: Final[int] = 60
ENDPOINT_TPM: Final[int] = 300000


class ModelSettingDict(TypedDict):
    id: str
    function_call_available: bool
    response_format_available: bool
    native_multimodal: bool
    context_length: int
    max_output_tokens: NotRequired[int]


# Moonshot models
MOONSHOT_DEFAULT_MODEL: Final[str] = "kimi-k2.5"
MOONSHOT_MODELS: Final[dict[str, ModelSettingDict]] = {
    "moonshot-v1-8k": {
        "id": "moonshot-v1-8k",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "moonshot-v1-32k": {
        "id": "moonshot-v1-32k",
        "context_length": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "moonshot-v1-128k": {
        "id": "moonshot-v1-128k",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "moonshot-v1-8k-vision-preview": {
        "id": "moonshot-v1-8k-vision-preview",
        "context_length": 8192,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "moonshot-v1-32k-vision-preview": {
        "id": "moonshot-v1-32k-vision-preview",
        "context_length": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "moonshot-v1-128k-vision-preview": {
        "id": "moonshot-v1-128k-vision-preview",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "kimi-latest": {
        "id": "kimi-latest",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "kimi-thinking-preview": {
        "id": "kimi-thinking-preview",
        "context_length": 131072,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "kimi-k2-0711-preview": {
        "id": "kimi-k2-0711-preview",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "kimi-k2-0905-preview": {
        "id": "kimi-k2-0905-preview",
        "context_length": 256000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "kimi-k2-turbo-preview": {
        "id": "kimi-k2-turbo-preview",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "kimi-k2-thinking": {
        "id": "kimi-k2-thinking",
        "context_length": 256000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "kimi-k2-thinking-turbo": {
        "id": "kimi-k2-thinking-turbo",
        "context_length": 256000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "kimi-k2.5": {
        "id": "kimi-k2.5",
        "context_length": 256000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
}

# Deepseek models
DEEPSEEK_DEFAULT_MODEL: Final[str] = "deepseek-chat"
DEEPSEEK_MODELS: Final[dict[str, ModelSettingDict]] = {
    "deepseek-chat": {
        "id": "deepseek-chat",
        "context_length": 64000,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "deepseek-reasoner": {
        "id": "deepseek-reasoner",
        "context_length": 64000,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": False,
    },
}

# Baichuan models
BAICHUAN_DEFAULT_MODEL: Final[str] = "Baichuan4"
BAICHUAN_MODELS: Final[dict[str, ModelSettingDict]] = {
    "Baichuan4": {
        "id": "Baichuan4",
        "context_length": 32768,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "Baichuan3-Turbo": {
        "id": "Baichuan3-Turbo",
        "context_length": 32768,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "Baichuan3-Turbo-128k": {
        "id": "Baichuan3-Turbo-128k",
        "context_length": 128000,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "Baichuan2-Turbo": {
        "id": "Baichuan2-Turbo",
        "context_length": 32768,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "Baichuan2-53B": {
        "id": "Baichuan2-53B",
        "context_length": 32768,
        "max_output_tokens": 2048,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
}

# Groq models
GROQ_DEFAULT_MODEL: Final[str] = "llama3-70b-8192"
GROQ_MODELS: Final[dict[str, ModelSettingDict]] = {
    "mixtral-8x7b-32768": {
        "id": "mixtral-8x7b-32768",
        "context_length": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "llama3-70b-8192": {
        "id": "llama3-70b-8192",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "llama3-8b-8192": {
        "id": "llama3-8b-8192",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "gemma-7b-it": {
        "id": "gemma-7b-it",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "gemma2-9b-it": {
        "id": "gemma2-9b-it",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "llama3-groq-70b-8192-tool-use-preview": {
        "id": "llama3-groq-70b-8192-tool-use-preview",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 8000,
        "native_multimodal": False,
    },
    "llama3-groq-8b-8192-tool-use-preview": {
        "id": "llama3-groq-8b-8192-tool-use-preview",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 8000,
        "native_multimodal": False,
    },
    "llama-3.1-70b-versatile": {
        "id": "llama-3.1-70b-versatile",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 8000,
        "native_multimodal": False,
    },
    "llama-3.1-8b-instant": {
        "id": "llama-3.1-8b-instant",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 8000,
        "native_multimodal": False,
    },
}

# Qwen models
QWEN_DEFAULT_MODEL: Final[str] = "qwen3.5-397b-a17b"
QWEN_MODELS: Final[dict[str, ModelSettingDict]] = {
    "qwen2.5-7b-instruct": {
        "id": "qwen2.5-7b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen2.5-14b-instruct": {
        "id": "qwen2.5-14b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen2.5-32b-instruct": {
        "id": "qwen2.5-32b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen2.5-coder-32b-instruct": {
        "id": "qwen2.5-coder-32b-instruct",
        "context_length": 30000,
        "max_output_tokens": 4096,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "qwq-32b": {
        "id": "qwq-32b",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "qwen2.5-72b-instruct": {
        "id": "qwen2.5-72b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen2-vl-72b-instruct": {
        "id": "qwen2-vl-72b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen2.5-vl-72b-instruct": {
        "id": "qwen2.5-vl-72b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen2.5-vl-7b-instruct": {
        "id": "qwen2.5-vl-7b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen2.5-vl-3b-instruct": {
        "id": "qwen2.5-vl-3b-instruct",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen-max": {
        "id": "qwen-max",
        "context_length": 8096,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen-max-longcontext": {
        "id": "qwen-max-longcontext",
        "context_length": 30000,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen-plus": {
        "id": "qwen-plus",
        "context_length": 131072,
        "max_output_tokens": 8096,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen-turbo": {
        "id": "qwen-turbo",
        "context_length": 8096,
        "max_output_tokens": 1500,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qvq-max": {
        "id": "qvq-max",
        "context_length": 122880,
        "max_output_tokens": 8192,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-235b-a22b": {
        "id": "qwen3-235b-a22b",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-32b": {
        "id": "qwen3-32b",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-30b-a3b": {
        "id": "qwen3-30b-a3b",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-14b": {
        "id": "qwen3-14b",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-8b": {
        "id": "qwen3-8b",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-4b": {
        "id": "qwen3-4b",
        "context_length": 131072,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-1.7b": {
        "id": "qwen3-1.7b",
        "context_length": 32768,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-0.6b": {
        "id": "qwen3-0.6b",
        "context_length": 32768,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-235b-a22b-instruct-2507": {
        "id": "qwen3-235b-a22b-instruct-2507",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-235b-a22b-thinking-2507": {
        "id": "qwen3-235b-a22b-thinking-2507",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-coder-480b-a35b-instruct": {
        "id": "qwen3-coder-480b-a35b-instruct",
        "context_length": 262144,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-next-80b-a3b-thinking": {
        "id": "qwen3-next-80b-a3b-thinking",
        "context_length": 159744,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-next-80b-a3b-instruct": {
        "id": "qwen3-next-80b-a3b-instruct",
        "context_length": 159744,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-max-preview": {
        "id": "qwen3-max-preview",
        "context_length": 262144,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-max": {
        "id": "qwen3-max",
        "context_length": 262144,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-vl-plus": {
        "id": "qwen3-vl-plus",
        "context_length": 262144,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-coder-plus": {
        "id": "qwen3-coder-plus",
        "context_length": 1000000,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-coder-flash": {
        "id": "qwen3-coder-flash",
        "context_length": 1000000,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "qwen3-vl-235b-a22b-thinking": {
        "id": "qwen3-vl-235b-a22b-thinking",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-235b-a22b-instruct": {
        "id": "qwen3-vl-235b-a22b-instruct",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-30b-a3b-thinking": {
        "id": "qwen3-vl-30b-a3b-thinking",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-30b-a3b-instruct": {
        "id": "qwen3-vl-30b-a3b-instruct",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-8b-thinking": {
        "id": "qwen3-vl-8b-thinking",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-8b-instruct": {
        "id": "qwen3-vl-8b-instruct",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-flash": {
        "id": "qwen3-vl-flash",
        "context_length": 262144,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-32b-thinking": {
        "id": "qwen3-vl-32b-thinking",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3-vl-32b-instruct": {
        "id": "qwen3-vl-32b-instruct",
        "context_length": 131072,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "qwen3.5-397b-a17b": {
        "id": "qwen3.5-397b-a17b",
        "context_length": 262144,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
}

# Yi models
YI_DEFAULT_MODEL: Final[str] = "yi-lightning"
YI_MODELS: Final[dict[str, ModelSettingDict]] = {
    "yi-lightning": {
        "id": "yi-lightning",
        "context_length": 16000,
        "max_output_tokens": 4096,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "yi-vision-v2": {
        "id": "yi-vision-v2",
        "context_length": 16000,
        "max_output_tokens": 2000,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
}

# ZhiPuAI models
ZHIPUAI_DEFAULT_MODEL: Final[str] = "glm-5"
ZHIPUAI_MODELS: Final[dict[str, ModelSettingDict]] = {
    "glm-4": {
        "id": "glm-4",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": False,
        "max_output_tokens": 4095,
        "native_multimodal": False,
    },
    "glm-4-plus": {
        "id": "glm-4-plus",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": False,
        "max_output_tokens": 4095,
        "native_multimodal": False,
    },
    "glm-4-0520": {
        "id": "glm-4-0520",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": False,
        "max_output_tokens": 4095,
        "native_multimodal": False,
    },
    "glm-4-air": {
        "id": "glm-4-air",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": False,
        "max_output_tokens": 4095,
        "native_multimodal": False,
    },
    "glm-4-airx": {
        "id": "glm-4-airx",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": False,
        "max_output_tokens": 4095,
        "native_multimodal": False,
    },
    "glm-4-flash": {
        "id": "glm-4-flash",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": False,
        "max_output_tokens": 4095,
        "native_multimodal": False,
    },
    "glm-4-long": {
        "id": "glm-4-long",
        "context_length": 1000000,
        "function_call_available": True,
        "response_format_available": False,
        "max_output_tokens": 4095,
        "native_multimodal": False,
    },
    "glm-4v": {
        "id": "glm-4v",
        "context_length": 2000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 1024,
        "native_multimodal": True,
    },
    "glm-4v-plus": {
        "id": "glm-4v-plus",
        "context_length": 2000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 1024,
        "native_multimodal": True,
    },
    "glm-4v-flash": {
        "id": "glm-4v-flash",
        "context_length": 2000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 1024,
        "native_multimodal": True,
    },
    "glm-zero-preview": {
        "id": "glm-zero-preview",
        "context_length": 16000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 12000,
        "native_multimodal": False,
    },
    "glm-4-alltools": {
        "id": "glm-4-alltools",
        "context_length": 128000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 20480,
        "native_multimodal": False,
    },
    "glm-z1-air": {
        "id": "glm-z1-air",
        "context_length": 32000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 32000,
        "native_multimodal": False,
    },
    "glm-z1-airx": {
        "id": "glm-z1-airx",
        "context_length": 32000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 30000,
        "native_multimodal": False,
    },
    "glm-z1-flash": {
        "id": "glm-z1-flash",
        "context_length": 32000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 32000,
        "native_multimodal": False,
    },
    "glm-4.1v-thinking-flashx": {
        "id": "glm-4.1v-thinking-flashx",
        "context_length": 64000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 16384,
        "native_multimodal": True,
    },
    "glm-4.1v-thinking-flash": {
        "id": "glm-4.1v-thinking-flash",
        "context_length": 64000,
        "function_call_available": False,
        "response_format_available": False,
        "max_output_tokens": 16384,
        "native_multimodal": True,
    },
    "glm-4.5": {
        "id": "glm-4.5",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 96000,
        "native_multimodal": False,
    },
    "glm-4.5-x": {
        "id": "glm-4.5-x",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 96000,
        "native_multimodal": False,
    },
    "glm-4.5-air": {
        "id": "glm-4.5-air",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 96000,
        "native_multimodal": False,
    },
    "glm-4.5-airx": {
        "id": "glm-4.5-airx",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 96000,
        "native_multimodal": False,
    },
    "glm-4.5-flash": {
        "id": "glm-4.5-flash",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 96000,
        "native_multimodal": False,
    },
    "glm-4.5v": {
        "id": "glm-4.5v",
        "context_length": 64000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 16384,
        "native_multimodal": True,
    },
    "glm-4.6": {
        "id": "glm-4.6",
        "context_length": 200000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 128000,
        "native_multimodal": False,
    },
    "glm-4.6v": {
        "id": "glm-4.6v",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 32000,
        "native_multimodal": True,
    },
    "glm-4.6v-flash": {
        "id": "glm-4.6v-flash",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 32000,
        "native_multimodal": True,
    },
    "glm-4.7": {
        "id": "glm-4.7",
        "context_length": 200000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 128000,
        "native_multimodal": False,
    },
    "glm-4.7-flash": {
        "id": "glm-4.7-flash",
        "context_length": 200000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 128000,
        "native_multimodal": False,
    },
    "glm-5": {
        "id": "glm-5",
        "context_length": 200000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 128000,
        "native_multimodal": False,
    },
    "glm-5-code": {
        "id": "glm-5-code",
        "context_length": 200000,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 128000,
        "native_multimodal": False,
    },
}

# Mistral models
MISTRAL_DEFAULT_MODEL: Final[str] = "mistral-small"
MISTRAL_MODELS: Final[dict[str, ModelSettingDict]] = {
    "open-mistral-7b": {
        "id": "open-mistral-7b",
        "context_length": 32000,
        "function_call_available": False,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "open-mixtral-8x7b": {
        "id": "open-mixtral-8x7b",
        "context_length": 32000,
        "function_call_available": False,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "open-mixtral-8x22b": {
        "id": "open-mixtral-8x22b",
        "context_length": 64000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "open-mistral-nemo": {
        "id": "open-mistral-nemo",
        "context_length": 128000,
        "function_call_available": False,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "codestral-latest": {
        "id": "codestral-latest",
        "context_length": 32000,
        "function_call_available": False,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "mistral-small-latest": {
        "id": "mistral-small-latest",
        "context_length": 30000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "mistral-medium-latest": {
        "id": "mistral-medium-latest",
        "context_length": 30000,
        "function_call_available": False,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "mistral-large-latest": {
        "id": "mistral-large-latest",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
}

# OpenAI models
OPENAI_DEFAULT_MODEL: Final[str] = "gpt-5.2"
OPENAI_MODELS: Final[dict[str, ModelSettingDict]] = {
    "gpt-35-turbo": {
        "id": "gpt-35-turbo",
        "context_length": 16385,
        "function_call_available": True,
        "response_format_available": True,
        "max_output_tokens": 4096,
        "native_multimodal": False,
    },
    "gpt-4-turbo": {
        "id": "gpt-4-turbo",
        "context_length": 128000,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "gpt-4": {
        "id": "gpt-4",
        "context_length": 8192,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "gpt-4o": {
        "id": "gpt-4o",
        "context_length": 128000,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-4o-mini": {
        "id": "gpt-4o-mini",
        "context_length": 128000,
        "max_output_tokens": 16384,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-4v": {
        "id": "gpt-4v",
        "context_length": 128000,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "o1-mini": {
        "id": "o1-mini",
        "context_length": 128000,
        "max_output_tokens": 65536,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "o1-preview": {
        "id": "o1-preview",
        "context_length": 128000,
        "max_output_tokens": 32768,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "o3": {
        "id": "o3",
        "context_length": 200000,
        "max_output_tokens": 100000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "o3-mini": {
        "id": "o3-mini",
        "context_length": 200000,
        "max_output_tokens": 100000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "o4-mini": {
        "id": "o4-mini",
        "context_length": 200000,
        "max_output_tokens": 100000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-4.1": {
        "id": "gpt-4.1",
        "context_length": 1047576,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-4.1-mini": {
        "id": "gpt-4.1-mini",
        "context_length": 1047576,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-4.1-nano": {
        "id": "gpt-4.1-nano",
        "context_length": 1047576,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5": {
        "id": "gpt-5",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5-mini": {
        "id": "gpt-5-mini",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5-nano": {
        "id": "gpt-5-nano",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5-chat-latest": {
        "id": "gpt-5-chat-latest",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5-codex": {
        "id": "gpt-5-codex",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5-pro": {
        "id": "gpt-5-pro",
        "context_length": 400000,
        "max_output_tokens": 272000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5.1": {
        "id": "gpt-5.1",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5.1-codex": {
        "id": "gpt-5.1-codex",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5.1-codex-mini": {
        "id": "gpt-5.1-codex-mini",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5.1-codex-max": {
        "id": "gpt-5.1-codex-max",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5.1-chat": {
        "id": "gpt-5.1-chat",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gpt-5.2": {
        "id": "gpt-5.2",
        "context_length": 400000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
}

# Anthropic models
ANTHROPIC_DEFAULT_MODEL: Final[str] = "claude-opus-4-6"
ANTHROPIC_MODELS: Final[dict[str, ModelSettingDict]] = {
    "claude-3-opus-20240229": {
        "id": "claude-3-opus-20240229",
        "context_length": 200000,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-3-sonnet-20240229": {
        "id": "claude-3-sonnet-20240229",
        "context_length": 200000,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "native_multimodal": True,
        "response_format_available": False,
    },
    "claude-3-haiku-20240307": {
        "id": "claude-3-haiku-20240307",
        "context_length": 200000,
        "max_output_tokens": 4096,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-3-5-haiku-20241022": {
        "id": "claude-3-5-haiku-20241022",
        "context_length": 200000,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "claude-3-5-sonnet-20240620": {
        "id": "claude-3-5-sonnet-20240620",
        "context_length": 200000,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-3-5-sonnet-20241022": {
        "id": "claude-3-5-sonnet-20241022",
        "context_length": 200000,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-3-7-sonnet-20250219": {
        "id": "claude-3-7-sonnet-20250219",
        "context_length": 200000,
        "max_output_tokens": 64000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-sonnet-4-20250514": {
        "id": "claude-sonnet-4-20250514",
        "context_length": 200000,
        "max_output_tokens": 64000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-opus-4-20250514": {
        "id": "claude-opus-4-20250514",
        "context_length": 200000,
        "max_output_tokens": 32000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-opus-4-1-20250805": {
        "id": "claude-opus-4-1-20250805",
        "context_length": 200000,
        "max_output_tokens": 32000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-sonnet-4-5-20250929": {
        "id": "claude-sonnet-4-5-20250929",
        "context_length": 200000,
        "max_output_tokens": 64000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-haiku-4-5-20251001": {
        "id": "claude-haiku-4-5-20251001",
        "context_length": 200000,
        "max_output_tokens": 64000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-opus-4-6": {
        "id": "claude-opus-4-6",
        "context_length": 200000,
        "max_output_tokens": 128000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "claude-sonnet-4-6": {
        "id": "claude-sonnet-4-6",
        "context_length": 200000,
        "max_output_tokens": 64000,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
}

# Minimax models
MINIMAX_DEFAULT_MODEL: Final[str] = "MiniMax-M2.5"
MINIMAX_MODELS: Final[dict[str, ModelSettingDict]] = {
    "MiniMax-M1": {
        "id": "MiniMax-M1",
        "context_length": 1000192,
        "max_output_tokens": 40000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "MiniMax-M2": {
        "id": "MiniMax-M2",
        "context_length": 204800,
        "max_output_tokens": 10240,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "MiniMax-M2-Stable": {
        "id": "MiniMax-M2-Stable",
        "context_length": 204800,
        "max_output_tokens": 10240,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "MiniMax-M2.1": {
        "id": "MiniMax-M2.1",
        "context_length": 204800,
        "max_output_tokens": 10240,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "MiniMax-M2.1-lightning": {
        "id": "MiniMax-M2.1-lightning",
        "context_length": 204800,
        "max_output_tokens": 10240,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "MiniMax-M2.1-highspeed": {
        "id": "MiniMax-M2.1-highspeed",
        "context_length": 204800,
        "max_output_tokens": 10240,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "MiniMax-M2.5": {
        "id": "MiniMax-M2.5",
        "context_length": 204800,
        "max_output_tokens": 10240,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "MiniMax-M2.5-highspeed": {
        "id": "MiniMax-M2.5-highspeed",
        "context_length": 204800,
        "max_output_tokens": 10240,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
}

# Gemini models
GEMINI_DEFAULT_MODEL: Final[str] = "gemini-3-flash"
GEMINI_MODELS: Final[dict[str, ModelSettingDict]] = {
    "gemini-2.5-pro": {
        "id": "gemini-2.5-pro",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-2.5-flash": {
        "id": "gemini-2.5-flash",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-2.5-flash-lite": {
        "id": "gemini-2.5-flash-lite",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-3-pro-preview": {
        "id": "gemini-3-pro-preview",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-3-pro": {
        "id": "gemini-3-pro",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-3-pro-image-preview": {
        "id": "gemini-3-pro-image-preview",
        "context_length": 65536,
        "max_output_tokens": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-3-flash-preview": {
        "id": "gemini-3-flash-preview",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-3-flash": {
        "id": "gemini-3-flash",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "gemini-3.1-pro-preview": {
        "id": "gemini-3.1-pro-preview",
        "context_length": 1048576,
        "max_output_tokens": 65536,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
}

# 百度文心一言 ERNIE 模型
ERNIE_DEFAULT_MODEL: Final[str] = "ernie-4.5-8k-preview"
ERNIE_MODELS: Final[dict[str, ModelSettingDict]] = {
    "ernie-lite": {
        "id": "ernie-lite",
        "context_length": 6144,
        "max_output_tokens": 2048,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "ernie-speed": {
        "id": "ernie-speed",
        "context_length": 126976,
        "max_output_tokens": 4096,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "ernie-speed-pro-128k": {
        "id": "ernie-speed-pro-128k",
        "context_length": 126976,
        "max_output_tokens": 4096,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": False,
    },
    "ernie-3.5-8k": {
        "id": "ernie-3.5-8k",
        "context_length": 5120,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "ernie-3.5-128k": {
        "id": "ernie-3.5-128k",
        "context_length": 126976,
        "max_output_tokens": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "ernie-4.0-8k-latest": {
        "id": "ernie-4.0-8k-latest",
        "context_length": 5120,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "ernie-4.0-8k": {
        "id": "ernie-4.0-8k",
        "context_length": 5120,
        "max_output_tokens": 2048,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "ernie-4.0-turbo-8k": {
        "id": "ernie-4.0-turbo-8k",
        "context_length": 5120,
        "max_output_tokens": 2048,
        "function_call_available": False,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "ernie-4.5-8k-preview": {
        "id": "ernie-4.5-8k-preview",
        "context_length": 5120,
        "max_output_tokens": 2048,
        "function_call_available": False,
        "response_format_available": True,
        "native_multimodal": False,
    },
}


STEPFUN_DEFAULT_MODEL: Final[str] = "step-3.5-flash"
STEPFUN_MODELS: Final[dict[str, ModelSettingDict]] = {
    "step-1-8k": {
        "id": "step-1-8k",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "step-1-32k": {
        "id": "step-1-32k",
        "context_length": 32000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "step-1-128k": {
        "id": "step-1-128k",
        "context_length": 128000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "step-1-256k": {
        "id": "step-1-256k",
        "context_length": 256000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "step-2-16k": {
        "id": "step-2-16k",
        "context_length": 16384,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "step-1-flash": {
        "id": "step-1-flash",
        "context_length": 8192,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "step-1v-8k": {
        "id": "step-1v-8k",
        "context_length": 8192,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "step-1v-32k": {
        "id": "step-1v-32k",
        "context_length": 32768,
        "function_call_available": False,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "step-2-mini": {
        "id": "step-2-mini",
        "context_length": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "step-3": {
        "id": "step-3",
        "context_length": 65536,
        "function_call_available": True,
        "response_format_available": False,
        "native_multimodal": True,
    },
    "step-3.5-flash": {
        "id": "step-3.5-flash",
        "context_length": 256000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
}


XAI_DEFAULT_MODEL: Final[str] = "grok-4"
XAI_MODELS: Final[dict[str, ModelSettingDict]] = {
    "grok-beta": {
        "id": "grok-beta",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "grok-2-latest": {
        "id": "grok-2-latest",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "grok-2-vision-latest": {
        "id": "grok-2-vision-latest",
        "context_length": 32768,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": True,
    },
    "grok-3-beta": {
        "id": "grok-beta",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "grok-3-fast-beta": {
        "id": "grok-beta",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "grok-3-mini-beta": {
        "id": "grok-beta",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "grok-3-mini-fast-beta": {
        "id": "grok-beta",
        "context_length": 131072,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
    "grok-4": {
        "id": "grok-4",
        "context_length": 256000,
        "function_call_available": True,
        "response_format_available": True,
        "native_multimodal": False,
    },
}



from __future__ import annotations

from typing import Any, TYPE_CHECKING

import httpx

from ..settings import normalize_settings
from ..types.enums import EmbeddingBackendType
from ..types.llm_parameters import EndpointSetting, ResponseMapping
from ..types.retrieval_parameters import EmbeddingData, EmbeddingResponse, EmbeddingUsage
from ..types.settings import SettingsDict
from ..utilities.gcp_token import get_token_with_cache
from ..retrieval_clients.common import BaseAsyncRetrievalClient, BaseRetrievalClient
from ..retrieval_clients.common import build_url, extract_json_path, render_template

if TYPE_CHECKING:
    from ..settings import Settings

DEFAULT_EMBEDDING_PROTOCOLS: dict[str, str] = {
    EmbeddingBackendType.OpenAI.value: "openai_embeddings",
    EmbeddingBackendType.Cohere.value: "cohere_embed_v2",
    EmbeddingBackendType.Jina.value: "openai_embeddings",
    EmbeddingBackendType.Voyage.value: "voyage_embeddings_v1",
    EmbeddingBackendType.Local.value: "openai_embeddings",
    EmbeddingBackendType.Custom.value: "custom_json_http",
}


def _normalize_backend_name(backend: EmbeddingBackendType | str) -> str:
    if isinstance(backend, EmbeddingBackendType):
        return backend.value
    return str(backend).lower()


def _ensure_list_input(value: str | list[str]) -> list[str]:
    if isinstance(value, str):
        return [value]
    return value


def _to_float_vector(value: Any) -> list[float]:
    if isinstance(value, list):
        return [float(v) for v in value]
    if isinstance(value, tuple):
        return [float(v) for v in value]
    raise ValueError(f"Invalid embedding vector type: {type(value)}")


def _parse_openai_style_embeddings(raw: dict[str, Any], model_id: str, inputs: list[str]) -> EmbeddingResponse:
    data = []
    for idx, item in enumerate(raw.get("data", [])):
        if not isinstance(item, dict):
            continue
        data.append(
            EmbeddingData(
                index=int(item.get("index", idx)),
                embedding=_to_float_vector(item.get("embedding", [])),
                text=inputs[idx] if idx < len(inputs) else None,
            )
        )

    usage_raw = raw.get("usage") if isinstance(raw.get("usage"), dict) else None
    usage = None
    if usage_raw is not None:
        usage = EmbeddingUsage(
            prompt_tokens=usage_raw.get("prompt_tokens"),
            total_tokens=usage_raw.get("total_tokens"),
        )

    return EmbeddingResponse(
        model=str(raw.get("model", model_id)),
        data=data,
        usage=usage,
        raw_response=raw,
    )


def _parse_cohere_embeddings(raw: dict[str, Any], model_id: str, inputs: list[str]) -> EmbeddingResponse:
    vectors: list[list[float]] = []

    embeddings = raw.get("embeddings")
    if isinstance(embeddings, dict):
        float_vectors = embeddings.get("float")
        if isinstance(float_vectors, list):
            vectors = [_to_float_vector(v) for v in float_vectors]
    elif isinstance(embeddings, list):
        vectors = [_to_float_vector(v) for v in embeddings]

    data = [EmbeddingData(index=idx, embedding=vector, text=inputs[idx] if idx < len(inputs) else None) for idx, vector in enumerate(vectors)]

    billed_units = raw.get("meta", {}).get("billed_units", {}) if isinstance(raw.get("meta"), dict) else {}
    prompt_tokens = billed_units.get("input_tokens") if isinstance(billed_units, dict) else None
    usage = EmbeddingUsage(prompt_tokens=prompt_tokens, total_tokens=prompt_tokens) if prompt_tokens is not None else None

    return EmbeddingResponse(
        model=str(raw.get("model", model_id)),
        data=data,
        usage=usage,
        raw_response=raw,
    )


def _usage_from_mapping(raw: dict[str, Any], response_mapping: ResponseMapping | None) -> EmbeddingUsage | None:
    if response_mapping is None:
        return None
    prompt_path = response_mapping.usage_map.get("prompt_tokens")
    total_path = response_mapping.usage_map.get("total_tokens")

    prompt_tokens = None
    if prompt_path:
        prompt_tokens = extract_json_path(raw, prompt_path)
        if prompt_tokens is None:
            raise ValueError(
                f"Embedding response_mapping.usage_map.prompt_tokens path '{prompt_path}' not found in response."
            )

    total_tokens = None
    if total_path:
        total_tokens = extract_json_path(raw, total_path)
        if total_tokens is None:
            raise ValueError(
                f"Embedding response_mapping.usage_map.total_tokens path '{total_path}' not found in response."
            )

    if prompt_tokens is None and total_tokens is None:
        return None

    try:
        parsed_prompt_tokens = int(prompt_tokens) if prompt_tokens is not None else None
    except (TypeError, ValueError) as exc:
        raise ValueError(
            "Embedding response_mapping.usage_map.prompt_tokens must resolve to an integer-compatible value."
        ) from exc

    try:
        parsed_total_tokens = int(total_tokens) if total_tokens is not None else None
    except (TypeError, ValueError) as exc:
        raise ValueError(
            "Embedding response_mapping.usage_map.total_tokens must resolve to an integer-compatible value."
        ) from exc

    return EmbeddingUsage(
        prompt_tokens=parsed_prompt_tokens,
        total_tokens=parsed_total_tokens,
    )


def _extract_item_field(item: Any, path: str | None):
    if path is None:
        return None
    if path.startswith("$"):
        return extract_json_path(item, path)
    return extract_json_path(item, f"$.{path}")


def _parse_custom_embeddings(raw: dict[str, Any], model_id: str, inputs: list[str], response_mapping: ResponseMapping | None) -> EmbeddingResponse:
    data_path = response_mapping.data_path if response_mapping and response_mapping.data_path else "$.data[*]"
    item_list = extract_json_path(raw, data_path)

    if item_list is None and response_mapping and response_mapping.data_path:
        raise ValueError(f"Embedding response_mapping.data_path '{data_path}' not found in response.")

    if item_list is None:
        item_list = []
    if not isinstance(item_list, list):
        item_list = [item_list]

    field_map = response_mapping.field_map if response_mapping is not None else {}
    response_data = []
    for idx, item in enumerate(item_list):
        mapped_index = _extract_item_field(item, field_map.get("index"))
        mapped_embedding = _extract_item_field(item, field_map.get("embedding"))
        mapped_text = _extract_item_field(item, field_map.get("text"))

        embedding_field_path = field_map.get("embedding")
        if embedding_field_path and mapped_embedding is None:
            raise ValueError(
                f"Embedding response_mapping.field_map.embedding path '{embedding_field_path}' not found for item[{idx}]."
            )

        if mapped_embedding is None:
            if isinstance(item, dict):
                mapped_embedding = item.get("embedding")
            elif isinstance(item, list):
                mapped_embedding = item

        if mapped_embedding is None:
            raise ValueError(
                f"Embedding item[{idx}] has no embedding vector. Provide field_map.embedding or ensure item contains 'embedding'."
            )

        index_field_path = field_map.get("index")
        if index_field_path and mapped_index is None:
            raise ValueError(
                f"Embedding response_mapping.field_map.index path '{index_field_path}' not found for item[{idx}]."
            )

        try:
            parsed_index = int(mapped_index) if mapped_index is not None else idx
        except (TypeError, ValueError) as exc:
            raise ValueError(
                f"Embedding item[{idx}] index must be integer-compatible, got: {mapped_index!r}."
            ) from exc

        try:
            parsed_embedding = _to_float_vector(mapped_embedding)
        except (TypeError, ValueError) as exc:
            raise ValueError(
                f"Embedding item[{idx}] vector must be a list/tuple of numeric values."
            ) from exc

        response_data.append(
            EmbeddingData(
                index=parsed_index,
                embedding=parsed_embedding,
                text=str(mapped_text) if mapped_text is not None else (inputs[idx] if idx < len(inputs) else None),
            )
        )

    model = model_id
    if response_mapping and response_mapping.model_path:
        mapped_model = extract_json_path(raw, response_mapping.model_path)
        if mapped_model is None:
            raise ValueError(
                f"Embedding response_mapping.model_path '{response_mapping.model_path}' not found in response."
            )
        model = str(mapped_model)

    if not response_data:
        raise ValueError(
            f"Embedding response produced no vectors after applying mapping. data_path='{data_path}'."
        )

    return EmbeddingResponse(
        model=model,
        data=response_data,
        usage=_usage_from_mapping(raw, response_mapping),
        raw_response=raw,
    )


def _default_auth_headers(endpoint: EndpointSetting) -> dict[str, str]:
    if endpoint.is_azure or endpoint.endpoint_type == "openai_azure":
        return {"api-key": endpoint.api_key or ""}
    if endpoint.endpoint_type == "openai_vertex":
        if endpoint.credentials is None:
            raise ValueError("OpenAI Vertex endpoint requires credentials")
        access_token, expires_at = get_token_with_cache(
            credentials=endpoint.credentials,
            proxy=endpoint.proxy,
            cached_token=endpoint.access_token,
            cached_expires_at=endpoint.access_token_expires_at,
        )
        endpoint.access_token = access_token
        endpoint.access_token_expires_at = expires_at
        return {"Authorization": f"Bearer {access_token}"}
    if endpoint.api_key:
        return {"Authorization": f"Bearer {endpoint.api_key}"}
    return {}


class EmbeddingClient(BaseRetrievalClient):
    def __init__(
        self,
        backend: EmbeddingBackendType | str,
        model: str | None = None,
        random_endpoint: bool = True,
        endpoint_id: str = "",
        http_client: httpx.Client | None = None,
        settings: Settings | SettingsDict | None = None,
    ):
        normalized_settings = normalize_settings(settings)
        backend_name = _normalize_backend_name(backend)
        backend_settings = normalized_settings.get_embedding_backend(backend_name)

        super().__init__(
            model=model or "",
            backend_name=backend_name,
            backend_settings=backend_settings,
            random_endpoint=random_endpoint,
            endpoint_id=endpoint_id,
            http_client=http_client,
            settings=normalized_settings,
        )

    def _switch_model(self, model: str):
        if model == self.model:
            return
        self.model = model
        self.model_setting = self.backend_settings.get_model_setting(model)
        self.model_id = self.model_setting.id
        self.endpoint = None

    def _request_json(
        self,
        *,
        endpoint: EndpointSetting,
        method: str,
        path: str,
        headers: dict[str, str] | None,
        body: dict | list | str | None,
        query: dict | None,
        timeout: float | httpx.Timeout | None,
    ) -> dict[str, Any]:
        request_headers = {"Content-Type": "application/json", **_default_auth_headers(endpoint)}
        if headers:
            request_headers.update(headers)

        client = self.http_client or httpx.Client(proxy=endpoint.proxy)
        should_close = self.http_client is None

        try:
            response = client.request(
                method=method.upper(),
                url=build_url(endpoint.api_base, path),
                headers=request_headers,
                params=query,
                json=body,
                timeout=timeout,
            )
            response.raise_for_status()
            return response.json()
        finally:
            if should_close:
                client.close()

    def _build_custom_request(
        self,
        *,
        model_id: str,
        input_value: str | list[str],
        dimensions: int | None,
        extra_body: dict[str, Any] | None,
    ) -> tuple[str, str, dict[str, str] | None, dict | list | str | None, dict | None]:
        mapping = self.model_setting.request_mapping
        method = mapping.method if mapping else "POST"
        path = mapping.path if mapping else "/embeddings"

        context = {
            "model": self.model,
            "model_id": model_id,
            "input": input_value,
            "inputs": _ensure_list_input(input_value),
            "dimensions": dimensions,
        }

        headers = render_template(mapping.headers, context) if mapping and mapping.headers else None
        body = render_template(mapping.body_template, context) if mapping and mapping.body_template is not None else None
        query = render_template(mapping.query_template, context) if mapping and mapping.query_template is not None else None

        if body is None:
            body = {
                "model": model_id,
                "input": input_value,
            }
        if dimensions is not None and isinstance(body, dict) and "dimensions" not in body:
            body["dimensions"] = dimensions
        if extra_body and isinstance(body, dict):
            body.update(extra_body)

        return method, path, headers, body, query

    def create_embeddings(
        self,
        *,
        input: str | list[str],
        model: str | None = None,
        dimensions: int | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> EmbeddingResponse:
        if model is not None:
            self._switch_model(model)

        endpoint, model_id = self._set_endpoint()
        protocol = self.model_setting.protocol or DEFAULT_EMBEDDING_PROTOCOLS.get(self.backend_name, "openai_embeddings")

        request_body_for_rate = {
            "input": input,
            "model": model_id,
            "dimensions": dimensions,
            "extra_body": extra_body,
        }
        self._acquire_rate_limit(endpoint, request_body_for_rate)

        if protocol == "openai_embeddings":
            if endpoint.is_azure or endpoint.endpoint_type == "openai_azure":
                path = f"/openai/deployments/{model_id}/embeddings"
                query = {"api-version": "2025-04-01-preview"}
                body: dict[str, Any] = {"input": input}
            else:
                path = "/embeddings"
                query = None
                body = {"model": model_id, "input": input}
            if dimensions is not None:
                body["dimensions"] = dimensions
            if extra_body:
                body.update(extra_body)

            raw = self._request_json(
                endpoint=endpoint,
                method="POST",
                path=path,
                headers=None,
                body=body,
                query=query,
                timeout=timeout,
            )
            return _parse_openai_style_embeddings(raw, model_id, _ensure_list_input(input))

        if protocol == "cohere_embed_v2":
            body = {"model": model_id, "texts": _ensure_list_input(input), "input_type": "search_document"}
            if extra_body:
                body.update(extra_body)
            raw = self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/v2/embed",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_cohere_embeddings(raw, model_id, _ensure_list_input(input))

        if protocol == "voyage_embeddings_v1":
            body = {"model": model_id, "input": input}
            if dimensions is not None:
                body["output_dimension"] = dimensions
            if extra_body:
                body.update(extra_body)
            raw = self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/embeddings",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_openai_style_embeddings(raw, model_id, _ensure_list_input(input))

        if protocol == "custom_json_http":
            method, path, headers, body, query = self._build_custom_request(
                model_id=model_id,
                input_value=input,
                dimensions=dimensions,
                extra_body=extra_body,
            )
            raw = self._request_json(
                endpoint=endpoint,
                method=method,
                path=path,
                headers=headers,
                body=body,
                query=query if isinstance(query, dict) else None,
                timeout=timeout,
            )
            return _parse_custom_embeddings(raw, model_id, _ensure_list_input(input), self.model_setting.response_mapping)

        raise ValueError(f"Unsupported embedding protocol: {protocol}")

    def embed(
        self,
        text: str,
        *,
        model: str | None = None,
        dimensions: int | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> list[float]:
        response = self.create_embeddings(
            input=text,
            model=model,
            dimensions=dimensions,
            extra_body=extra_body,
            timeout=timeout,
        )
        if not response.data:
            return []
        return response.data[0].embedding

    def embed_batch(
        self,
        texts: list[str],
        *,
        model: str | None = None,
        dimensions: int | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> list[list[float]]:
        response = self.create_embeddings(
            input=texts,
            model=model,
            dimensions=dimensions,
            extra_body=extra_body,
            timeout=timeout,
        )
        return [item.embedding for item in response.data]


class AsyncEmbeddingClient(BaseAsyncRetrievalClient):
    def __init__(
        self,
        backend: EmbeddingBackendType | str,
        model: str | None = None,
        random_endpoint: bool = True,
        endpoint_id: str = "",
        http_client: httpx.AsyncClient | None = None,
        settings: Settings | SettingsDict | None = None,
    ):
        normalized_settings = normalize_settings(settings)
        backend_name = _normalize_backend_name(backend)
        backend_settings = normalized_settings.get_embedding_backend(backend_name)

        super().__init__(
            model=model or "",
            backend_name=backend_name,
            backend_settings=backend_settings,
            random_endpoint=random_endpoint,
            endpoint_id=endpoint_id,
            http_client=http_client,
            settings=normalized_settings,
        )

    def _switch_model(self, model: str):
        if model == self.model:
            return
        self.model = model
        self.model_setting = self.backend_settings.get_model_setting(model)
        self.model_id = self.model_setting.id
        self.endpoint = None

    async def _request_json(
        self,
        *,
        endpoint: EndpointSetting,
        method: str,
        path: str,
        headers: dict[str, str] | None,
        body: dict | list | str | None,
        query: dict | None,
        timeout: float | httpx.Timeout | None,
    ) -> dict[str, Any]:
        request_headers = {"Content-Type": "application/json", **_default_auth_headers(endpoint)}
        if headers:
            request_headers.update(headers)

        client = self.http_client or httpx.AsyncClient(proxy=endpoint.proxy)
        should_close = self.http_client is None

        try:
            response = await client.request(
                method=method.upper(),
                url=build_url(endpoint.api_base, path),
                headers=request_headers,
                params=query,
                json=body,
                timeout=timeout,
            )
            response.raise_for_status()
            return response.json()
        finally:
            if should_close:
                await client.aclose()

    def _build_custom_request(
        self,
        *,
        model_id: str,
        input_value: str | list[str],
        dimensions: int | None,
        extra_body: dict[str, Any] | None,
    ) -> tuple[str, str, dict[str, str] | None, dict | list | str | None, dict | None]:
        mapping = self.model_setting.request_mapping
        method = mapping.method if mapping else "POST"
        path = mapping.path if mapping else "/embeddings"

        context = {
            "model": self.model,
            "model_id": model_id,
            "input": input_value,
            "inputs": _ensure_list_input(input_value),
            "dimensions": dimensions,
        }

        headers = render_template(mapping.headers, context) if mapping and mapping.headers else None
        body = render_template(mapping.body_template, context) if mapping and mapping.body_template is not None else None
        query = render_template(mapping.query_template, context) if mapping and mapping.query_template is not None else None

        if body is None:
            body = {
                "model": model_id,
                "input": input_value,
            }
        if dimensions is not None and isinstance(body, dict) and "dimensions" not in body:
            body["dimensions"] = dimensions
        if extra_body and isinstance(body, dict):
            body.update(extra_body)

        return method, path, headers, body, query

    async def create_embeddings(
        self,
        *,
        input: str | list[str],
        model: str | None = None,
        dimensions: int | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> EmbeddingResponse:
        if model is not None:
            self._switch_model(model)

        endpoint, model_id = self._set_endpoint()
        protocol = self.model_setting.protocol or DEFAULT_EMBEDDING_PROTOCOLS.get(self.backend_name, "openai_embeddings")

        request_body_for_rate = {
            "input": input,
            "model": model_id,
            "dimensions": dimensions,
            "extra_body": extra_body,
        }
        await self._acquire_rate_limit(endpoint, request_body_for_rate)

        if protocol == "openai_embeddings":
            if endpoint.is_azure or endpoint.endpoint_type == "openai_azure":
                path = f"/openai/deployments/{model_id}/embeddings"
                query = {"api-version": "2025-04-01-preview"}
                body: dict[str, Any] = {"input": input}
            else:
                path = "/embeddings"
                query = None
                body = {"model": model_id, "input": input}
            if dimensions is not None:
                body["dimensions"] = dimensions
            if extra_body:
                body.update(extra_body)

            raw = await self._request_json(
                endpoint=endpoint,
                method="POST",
                path=path,
                headers=None,
                body=body,
                query=query,
                timeout=timeout,
            )
            return _parse_openai_style_embeddings(raw, model_id, _ensure_list_input(input))

        if protocol == "cohere_embed_v2":
            body = {"model": model_id, "texts": _ensure_list_input(input), "input_type": "search_document"}
            if extra_body:
                body.update(extra_body)
            raw = await self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/v2/embed",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_cohere_embeddings(raw, model_id, _ensure_list_input(input))

        if protocol == "voyage_embeddings_v1":
            body = {"model": model_id, "input": input}
            if dimensions is not None:
                body["output_dimension"] = dimensions
            if extra_body:
                body.update(extra_body)
            raw = await self._request_json(
                endpoint=endpoint,
                method="POST",
                path="/embeddings",
                headers=None,
                body=body,
                query=None,
                timeout=timeout,
            )
            return _parse_openai_style_embeddings(raw, model_id, _ensure_list_input(input))

        if protocol == "custom_json_http":
            method, path, headers, body, query = self._build_custom_request(
                model_id=model_id,
                input_value=input,
                dimensions=dimensions,
                extra_body=extra_body,
            )
            raw = await self._request_json(
                endpoint=endpoint,
                method=method,
                path=path,
                headers=headers,
                body=body,
                query=query if isinstance(query, dict) else None,
                timeout=timeout,
            )
            return _parse_custom_embeddings(raw, model_id, _ensure_list_input(input), self.model_setting.response_mapping)

        raise ValueError(f"Unsupported embedding protocol: {protocol}")

    async def embed(
        self,
        text: str,
        *,
        model: str | None = None,
        dimensions: int | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> list[float]:
        response = await self.create_embeddings(
            input=text,
            model=model,
            dimensions=dimensions,
            extra_body=extra_body,
            timeout=timeout,
        )
        if not response.data:
            return []
        return response.data[0].embedding

    async def embed_batch(
        self,
        texts: list[str],
        *,
        model: str | None = None,
        dimensions: int | None = None,
        extra_body: dict[str, Any] | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> list[list[float]]:
        response = await self.create_embeddings(
            input=texts,
            model=model,
            dimensions=dimensions,
            extra_body=extra_body,
            timeout=timeout,
        )
        return [item.embedding for item in response.data]


__all__ = [
    "EmbeddingClient",
    "AsyncEmbeddingClient",
]



# @Author: Bi Ying
# @Date:   2024-07-26 14:48:55
from ..types.enums import BackendType
from .openai_compatible_client import OpenAICompatibleChatClient, AsyncOpenAICompatibleChatClient


class LocalChatClient(OpenAICompatibleChatClient):
    DEFAULT_MODEL = ""
    BACKEND_NAME = BackendType.Local


class AsyncLocalChatClient(AsyncOpenAICompatibleChatClient):
    DEFAULT_MODEL = ""
    BACKEND_NAME = BackendType.Local



import importlib
from typing import Any, cast

from pydantic import BaseModel

from ..chat_clients.utils import get_token_counts
from ..settings import settings


_SERVER_IMPORT_ERROR: ModuleNotFoundError | None = None
_uvicorn: Any = None
_fastapi: Any = None
_http_exception_cls: type[Exception] = RuntimeError

try:
    _uvicorn = importlib.import_module("uvicorn")
    _fastapi = importlib.import_module("fastapi")
    _http_exception_cls = _fastapi.HTTPException
except ModuleNotFoundError as import_error:  # pragma: no cover - import guard
    _SERVER_IMPORT_ERROR = import_error


def _ensure_server_dependencies() -> None:
    if _SERVER_IMPORT_ERROR is not None:
        raise ModuleNotFoundError(
            "FastAPI server dependencies are not installed. "
            "Install with: pip install vv-llm[server]"
        ) from _SERVER_IMPORT_ERROR


def _create_fastapi_app() -> Any:
    _ensure_server_dependencies()
    return _fastapi.FastAPI()


token_server = _create_fastapi_app() if _SERVER_IMPORT_ERROR is None else None


class TokenCountRequest(BaseModel):
    text: str | dict
    model: str = ""


if token_server is not None:

    @token_server.post("/count_tokens")
    async def count_tokens(request: TokenCountRequest):
        try:
            token_count = get_token_counts(request.text, request.model, use_token_server_first=False)
            return {"total_tokens": token_count}
        except Exception as e:
            raise cast(Any, _http_exception_cls)(status_code=500, detail=str(e)) from None


def run_token_server(host: str | None = None, port: int | None = None):
    """
    启动一个简单的HTTP服务器来处理token计数请求。参数均留空则使用 settings.token_server 的配置。

    参数:
        host (str): 服务器主机地址。
        port (int): 服务器端口。
    """
    _ensure_server_dependencies()

    if host is None or port is None:
        if settings.token_server is None:
            raise ValueError("Token server is not enabled.")

        _host = settings.token_server.host
        _port = settings.token_server.port
    else:
        _host = host
        _port = port

    _uvicorn.run(token_server, host=_host, port=_port)


if __name__ == "__main__":
    run_token_server()



from ..types.enums import BackendType
from ..types.defaults import MINIMAX_DEFAULT_MODEL
from .openai_compatible_client import OpenAICompatibleChatClient, AsyncOpenAICompatibleChatClient


class MiniMaxChatClient(OpenAICompatibleChatClient):
    DEFAULT_MODEL = MINIMAX_DEFAULT_MODEL
    BACKEND_NAME = BackendType.MiniMax


class AsyncMiniMaxChatClient(AsyncOpenAICompatibleChatClient):
    DEFAULT_MODEL = MINIMAX_DEFAULT_MODEL
    BACKEND_NAME = BackendType.MiniMax
